package path;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import javax.swing.JFrame;
import javax.swing.JPanel;
import test.Test;

public class Game extends Canvas implements Runnable {
	private static final long serialVersionUID = 1L;
	public static final int SCALE = 4;
	public static final int WIDTH = 640;
	public static final int HEIGHT = 480;
	public static final int SCALED_WIDTH = WIDTH / SCALE;
	public static final int SCALED_HEIGHT = HEIGHT / SCALE;
	public volatile boolean isRunning;
	
	private BufferedImage image;
	private int[] data;
	private Test test;
	
	public Game() {
		isRunning = false;
		//area = new Area(640 / SCALE, 480 / SCALE);
		Dimension size = new Dimension(WIDTH, HEIGHT);
		setSize(size);
		
		image = new BufferedImage(SCALED_WIDTH, SCALED_HEIGHT, BufferedImage.TYPE_INT_ARGB);
		data = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
		
		test = new Test();
		addMouseListener(test.input);
		addMouseMotionListener(test.input);
	}
	
	public synchronized void start() {
		isRunning = true;
		Thread thread = new Thread(this);
		thread.start();
		try {
			Thread.currentThread().join();
		}
		catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void run() {
		
		long now;
		long lastTime = System.nanoTime();;
		double unprocessed = 0.0;
		final double nsPerTick = 1000000000.0 / 60.0;
		while (isRunning) {
			now = System.nanoTime();
			unprocessed += (now - lastTime) / nsPerTick;
			lastTime = now;
			
			while (unprocessed >= 1.0) {
				unprocessed -= 1.0;
				tick();
			}
			render();
			try {
				Thread.sleep(1);
			}
			catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	public void tick() {
		test.tick();
	}
	
	public void render() {
		BufferStrategy bs = getBufferStrategy();
		if (bs == null) {
			createBufferStrategy(3);
			return;
		}
		
		test.render(data);
		
		Graphics g = bs.getDrawGraphics();
		g.setColor(Color.gray);
		g.fillRect(0, 0, WIDTH, HEIGHT);
		g.drawImage(image, 0, 0, WIDTH, HEIGHT, null);
		g.dispose();
		bs.show();
		
	}
	
	public static void main(String[] args) {
		Game game = new Game();
		JFrame frame = new JFrame("A* Pathfinding Test");
		frame.add(new JPanel().add(game));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLocationRelativeTo(null);
		frame.pack();
		frame.setVisible(true);
		
		game.start();
	}
}
